require 'test_helper'

class MessageTest < EssentialTest
  include Essential::Messaging

  def setup
    @channel = Channel.list.first
    assert_equal 'default', @channel.name, 'default channel must exist'
    @subscriber = @channel.subscribers.first
    @transport = Transport.retrieve(@subscriber.transport_sid)
  end

  def test_create
    required = [
      {body: 'a body', subscriber: @subscriber.sid},
      {body: 'a body', subscriber: @subscriber.phone_number}
    ]

    optional = [
      {},
      {transport: @transport.sid},
      {transport: @transport.endpoint},
      {channel:   @channel.sid},
      {channel:   @channel.name},
    ]

    required.each do |req|
      optional.each do |opt|
        assert_valid_message Message.create(req.merge(opt))
      end
    end

  end

  def test_create_errors
    map_of_brokenness = {
      nil => [:body, :subscriber],

      {} => [:body, :subscriber],

      { body: 'haha'} => [:subscriber],

      { body: 'haha', subscriber: 'not_a_sid'} => [:subscriber],

      { body: 'haha',
        subscriber: 'not_a_phone_number' } => [:subscriber],

      { body: 'haha',
        subscriber: @subscriber.sid,
        transport: 'not_a_sid' } => [:transport],

      { body: 'haha',
        subscriber: @subscriber.sid,
        transport: 'not_a_transport' } => [:transport],

      { body: 'haha',
        subscriber: @subscriber.sid,
        channel: 'not_a_sid' } => [:channel],

      { body: 'haha',
        subscriber_sid: @subscriber.sid,
        channel: 'olives' } => [:channel],
    }

    map_of_brokenness.each do |params, errors|
      assert_error_with_params *errors do
        Message.create(params)
      end
    end
  end

  def test_create_via_aliases
    assert_valid_message Message.create(
                           subscriber: @subscriber.phone_number,
                           body: 'addressed to @subscriber.phone_number'
                         )

    assert_valid_message Message.create(
                           subscriber: @subscriber.sid,
                           transport: @transport.endpoint,
                           body: 'sent via @transport.endpoint'
                         )

    assert_valid_message Message.create(
                           subscriber: @subscriber.phone_number,
                           channel: @channel.name,
                           body: 'sent via @channel.name'
                         )
  end

  def test_create_new_subscriber
    next_phone = @channel.subscribers.map(&:phone_number).sort.last.to_i + 1

    m = Message.create(
      subscriber: format('%d', next_phone),
      body: format('a message for %d', next_phone)
    )

    refute_equal m.subscriber_sid, @subscriber.sid, 'subscriber_sid must match'
    assert_equal m.transport_sid, @transport.sid, 'transport_sid must match'
    assert_equal m.channel_sid, @channel.sid, 'channel_sid must match'
  end

  def test_relations
    message = @subscriber.messages.last

    assert_equal @channel,    message.channel
    assert_equal @subscriber, message.subscriber
    assert_equal @transport,  message.transport
  end

  protected

  def assert_valid_message(message)
    assert_equal message.subscriber_sid, @subscriber.sid, 'subscriber_sid must match'
    assert_equal message.transport_sid, @transport.sid, 'transport_sid must match'
    assert_equal message.channel_sid, @channel.sid, 'channel_sid must match'
  end
end
