require 'test_helper'

class SubscriberTest < EssentialTest
  include Essential::Messaging

  def setup
    @channel = Channel.list.first
  end

  def test_create
    sub = Subscriber.create(
      phone_number: '1002003000',
      channel: @channel.name
    )

    assert_equal '1002003000', sub.phone_number
    assert_equal @channel.sid, sub.channel_sid
  end

  def test_list
    (1..4).each do |n|
      Subscriber.create(phone_number: format('100200300%d', n))
    end

    subs = Subscriber.list
    assert subs.size > 4, 'should have more than 4 subscribers'

    sub_sids = Set.new(subs.map(&:sid))
    assert_equal subs.size, sub_sids.size, 'must contain N unique sids'
  end

  def test_list_bad_channel_sid
    error = assert_raises Essential::APIError do
      Subscriber.list(params: {channel: :not_a_real_sid})
    end

    assert_equal 400, error.http_status
    refute_nil error.params
    assert error.params.key?('channel'), 'must complain about channel_sid'
  end

  def test_list_bad_channel_name
    error = assert_raises Essential::APIError do
      Subscriber.list(params: {channel: :not_a_real_name})
    end

    assert_equal 400, error.http_status
    refute_nil error.params
    assert error.params.key?('channel'), 'must complain about channel_name'
  end

  def test_update
    sub = Subscriber.create(
      phone_number: '9998887777',
      channel: @channel.name
    )
    sid = sub.sid.dup

    sub.update(phone_number: '7778889999')
    assert_equal sid, sub.sid
    assert_equal '7778889999', sub.phone_number

    sub = Subscriber.retrieve(sid)
    assert_equal sid, sub.sid
    assert_equal '7778889999', sub.phone_number

    error = assert_raises Essential::APIError do
      duplicate = Subscriber.create(
        phone_number: '1012023333',
        channel: @channel.sid
      )
      duplicate.update(phone_number: sub.phone_number)
    end
    assert_equal 400, error.http_status
    refute_nil error.params
    assert error.params.key?('phone_number')

    # change it back: if we're running a smoke test, these are
    # unique and we need to make sure that '7778889999' doesn't
    # exist at the start of the next run!
    sub.update(phone_number: '9998887777')
    assert_equal '9998887777', sub.phone_number
  end

  def test_unsubscribe
    sub = Subscriber.create(
      phone_number: '1002003000',
      channel: @channel.name
    )

    response = sub.unsubscribe

    Subscriber.retrieve(sub.sid)
  end

  def test_relations
    sub = @channel.subscribers.first

    assert_equal @channel, sub.channel
    assert_equal sub.transport_sid, sub.transport.sid

    mess = sub.messages
    refute_equal 0, mess.count
    assert mess.all? {|m| Message === m}
  end
end
